<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use App\Traits\MobileTrait;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\Validator;
use App\Models\Gift;
use App\Traits\WebTrait;
use Illuminate\Support\Facades\Http;

use Illuminate\Support\Facades\Storage;
use Illuminate\Http\File;

class GiftController extends Controller
{
    use WebTrait;

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    //  private function fileNameExists($fileName)
    //     {
    //         return Storage::disk('public')->exists($fileName);
    //     }

    public function index()
    {
        $gifts = Gift::orderBy('price', 'ASC')->get();

        return $this->data($gifts);
    }

    public function store(Request $request )
    {
        $validator = Validator::make($request->all(), [
            'name' => ['required', 'string', 'min:2', 'max:255'],
            'cover_image_url' => 'required|url',
            'svga' => ['required', 'max:2048'],
            'price' => ['required', 'numeric', 'min:0'],
            'type' => ['in:0,1,2'],
            'category_gift_id' => 'required|exists:category_gifts,id',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 400);
        }

        // Download and save the image temporarily
        $tempImagePath = tempnam(sys_get_temp_dir(), 'cover_image');
        $imageContent = Http::get($request->cover_image_url)->body();
        file_put_contents($tempImagePath, $imageContent);

        // Upload the image
        $cover = Storage::disk('public')->putFile('images/gifts', new File($tempImagePath));
        $request->merge(['cover' => $cover]);

        //Svga
        $uniqueName = $this->generateUniqueFileName();
        $fileName = $uniqueName . '.svga';
        $svga = $request->file('svga_image')->storeAs('/public/svgas/gifts', $fileName);
        $fullFileName = 'svgas/gifts/' . $fileName;
        $request->request->add(['svga' => $fullFileName]);

        try{

            $gift = Gift::create($request->all());
            return response()->json(['message'=>'تم تسجيل الهدية','gift'=>$gift]); //Success Insert
        }catch(QueryException $e){
            return $this->error('E200', ''); //DB err(General)
        }

    }

    public function getUserGifts($id)
    {
        $user = User::findOrFail($id);

        return $this->data($user->gifts);
    }

    protected function generateUniqueFileName()
    {
        $length = 20;
        $chars='0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $totalChars = strlen($chars);
        $totalRepeat = ceil($length/$totalChars);
        $repeatString = str_repeat($chars, $totalRepeat);
        $shuffleString = str_shuffle($repeatString);
        $fileName = substr($shuffleString,1,$length);

        if($this->fileNameExists($fileName)){
            $this->generateUniqueFileName();
        }

        return $fileName;
    }


}
